
-- drop before creation
DROP TABLE IF EXISTS COMPONENTS;
DROP TABLE IF EXISTS ASSOCIATIONS;
DROP TABLE IF EXISTS RESOURCES;
DROP TABLE IF EXISTS COMPONENT_TYPES;
DROP TABLE IF EXISTS ASSOCIATION_TYPES;
DROP TABLE IF EXISTS RESOURCE_TYPES;
DROP TABLE IF EXISTS OBJECTS_REPOSITORY;

DROP INDEX IF EXISTS COMPONENTS_NAME_PATH_TYPE;
DROP INDEX IF EXISTS COMPONENTS_TYPE_DECL_PATH;
DROP INDEX IF EXISTS ASSOCIATIONS_TYPE_TO;
DROP INDEX IF EXISTS ASSOCIATIONS_TYPE_FROM;

-- model tables
CREATE CACHED TABLE COMPONENTS (
		ID INTEGER NOT NULL IDENTITY PRIMARY KEY,
		NAME VARCHAR(5000) NOT NULL,
		PATH VARCHAR(5000) NOT NULL,
		DISPLAY_NAME VARCHAR(5000) NOT NULL,
		DISPLAY_PATH VARCHAR(5000) NOT NULL,  
	    DECLARATION_STATUS CHAR(1) NOT NULL,
		QUERY_DATA VARCHAR(5000),     
		SRC_RESOURCE_ID INTEGER ,
		SRC_START_POS INTEGER,
		SRC_END_POS INTEGER,
		COMP_TYPE_ID SMALLINT NOT NULL,
		OBJECT BLOB NOT NULL
	);

CREATE INDEX COMPONENTS_NAME_PATH_TYPE ON COMPONENTS(NAME, PATH, COMP_TYPE_ID);
CREATE INDEX COMPONENTS_TYPE_DECL_PATH ON COMPONENTS(COMP_TYPE_ID, DECLARATION_STATUS, PATH);


CREATE CACHED TABLE ASSOCIATIONS (
		ID INTEGER NOT NULL IDENTITY PRIMARY KEY,
		SRC_RESOURCE_ID INTEGER ,
		SRC_START_POS INTEGER,
		SRC_END_POS INTEGER,
		FROM_COMP_ID INTEGER NOT NULL,
		TO_COMP_ID INTEGER NOT NULL,
		ASSOC_TYPE_ID SMALLINT NOT NULL,
		OBJECT BLOB NOT NULL
	);

-- CREATE INDEX ASSOCIATIONS_TYPE_TO ON ASSOCIATIONS(ASSOC_TYPE_ID, TO_COMP_ID);
-- CREATE INDEX ASSOCIATIONS_TYPE_FROM ON ASSOCIATIONS(ASSOC_TYPE_ID, FROM_COMP_ID);

ALTER TABLE ASSOCIATIONS ADD CONSTRAINT ASSOCIATIONS_FROM_COMP FOREIGN KEY(FROM_COMP_ID) REFERENCES COMPONENTS(ID) ;
--	ON DELETE CASCADE
--	ON UPDATE CASCADE;

ALTER TABLE ASSOCIATIONS ADD CONSTRAINT ASSOCIATIONS_TO_COMP FOREIGN KEY(TO_COMP_ID) REFERENCES COMPONENTS(ID) ;
--	ON DELETE CASCADE
--	ON UPDATE CASCADE;

CREATE CACHED TABLE RESOURCES (
		ID INTEGER NOT NULL IDENTITY PRIMARY KEY,
		NAME VARCHAR(5000) NOT NULL,
		PATH VARCHAR(5000) NOT NULL,
		LAST_UPDATE TIMESTAMP NOT NULL,
		RSRC_TYPE_ID SMALLINT NOT NULL,
		OBJECT BLOB NOT NULL
	);
	
CREATE INDEX RESOURCES_NAME_PATH_TYPE ON RESOURCES(NAME, PATH, RSRC_TYPE_ID);

ALTER TABLE ASSOCIATIONS ADD CONSTRAINT ASSOCIATIONS_SRC_RESOURCE_ID FOREIGN KEY(SRC_RESOURCE_ID) REFERENCES RESOURCES(ID); 
--	ON DELETE CASCADE
--	ON UPDATE CASCADE;

ALTER TABLE COMPONENTS ADD CONSTRAINT COMPONENTS_SRC_RESOURCE_ID FOREIGN KEY(SRC_RESOURCE_ID) REFERENCES RESOURCES(ID) ;
--	ON DELETE CASCADE
--	ON UPDATE CASCADE;


-- meta model tables

CREATE MEMORY TABLE COMPONENT_TYPES (
		ID SMALLINT NOT NULL PRIMARY KEY,
		NAME VARCHAR(50) NOT NULL,
		DESCRIPTION VARCHAR(500),
		PARENT SMALLINT,
		WEIGHT REAL NOT NULL DEFAULT 0.0
	);

ALTER TABLE COMPONENTS ADD CONSTRAINT COMPONENTS_COMP_TYPE_ID FOREIGN KEY(COMP_TYPE_ID) REFERENCES COMPONENT_TYPES(ID) ;
--	ON DELETE CASCADE
--	ON UPDATE CASCADE;

CREATE MEMORY TABLE ASSOCIATION_TYPES (
		ID SMALLINT NOT NULL PRIMARY KEY,
		NAME VARCHAR(50) NOT NULL,
		DESCRIPTION VARCHAR(500),
		PARENT SMALLINT,
		WEIGHT REAL NOT NULL DEFAULT 0.0
	);

ALTER TABLE ASSOCIATIONS ADD CONSTRAINT ASSOCIATIONS_ASSOC_TYPE_ID FOREIGN KEY(ASSOC_TYPE_ID) REFERENCES ASSOCIATION_TYPES(ID) ;
--	ON DELETE CASCADE
--	ON UPDATE CASCADE;

CREATE MEMORY TABLE RESOURCE_TYPES (
		ID SMALLINT NOT NULL PRIMARY KEY,
		NAME VARCHAR(50) NOT NULL,
		DESCRIPTION VARCHAR(500),
		PARENT SMALLINT,
		WEIGHT REAL NOT NULL DEFAULT 0.0
	);

ALTER TABLE RESOURCES ADD CONSTRAINT RESOURCES_RSRC_TYPE_ID FOREIGN KEY(RSRC_TYPE_ID) REFERENCES RESOURCE_TYPES(ID) ;
--	ON DELETE CASCADE
--	ON UPDATE CASCADE;

CREATE CACHED TABLE OBJECTS_REPOSITORY (
		NAME VARCHAR(5000) NOT NULL PRIMARY KEY,
		CLASS VARCHAR(5000) NOT NULL,
		LAST_UPDATE TIMESTAMP NOT NULL,
		OBJECT BLOB NOT NULL
	);


-- init full text search
-- CREATE ALIAS IF NOT EXISTS FTL_INIT FOR "com.nwire.studio.core.h2ext.FullTextLucene.init"; 
-- CALL FTL_INIT(); 

-- set up indexing on components table
-- CALL FTL_CREATE_INDEX('PUBLIC', 'COMPONENTS', 'NAME'); 	
